package com.fg.scripting;

import org.springframework.aop.RawTargetAccess;

/**
 * Groovy proxy interface declares method available for wrap and unwrap inner scripting instance, and set instance initializer
 * that initializes instance after eventual inner instance reinstantiation when source code changes.
 *
 * @author Jan Novotný, FG Forrest a.s. (c) 2007
 * @version $Id: ScriptingProxy.java,v 1.2 2009/11/15 20:05:12 u_novoj Exp $
 */
public interface ScriptingProxy extends RawTargetAccess {

	/**
	 * Sets instance initalizer that initializes reinstantiated object after source code change.
	 * @param factory for initializing new inner scripting instance
	 */
	void setInstanceInitializer(ProxyInstanceFactory factory);

	/**
	 * Returns set instance initializer.
	 * @see #setInstanceInitializer(ProxyInstanceFactory)
	 * @return instance initializer that should be used to newly created inner scripting instance
	 */
	ProxyInstanceFactory getInstanceInitializer();

	/**
	 * Sets inner scripting instance.
	 * @param scriptingInstance to wrap in this proxy
	 */
	void wrap(Object scriptingInstance);

	/**
	 * Returns inner scripting instance.
	 * Please do not store reference to this instance in any long living scope (static variables, session, servlet
	 * context, singletons and so on)
	 * @return pure scripting instance hidden inside this proxy
	 */
	Object unwrap();

	/**
	 * Performs up to date check and potentialy updated outdated inner instance.
	 *
	 * @return false when inner scripting instance was updated
	 */
	boolean isUpToDate();

}
