package com.fg.scripting.groovy.classloader;

import groovy.lang.GroovyClassLoader;
import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.springframework.context.support.AbstractRefreshableApplicationContext;

import java.lang.reflect.Method;

/**
 * Description
 *
 * @author Jan Novotný, FG Forrest a.s. (c) 2007
 * @version $Id: UptodateSourceCheckingAdvice.java,v 1.3 2009/11/15 20:05:11 u_novoj Exp $
 */
public class UptodateSourceCheckingAdvice implements MethodInterceptor {
	private final AbstractRefreshableApplicationContext applicationContext;
	private final GroovyClassLoader groovyClassLoader;
	private final String beanName;

	public UptodateSourceCheckingAdvice(AbstractRefreshableApplicationContext applicationContext, GroovyClassLoader groovyClassLoader, String beanName) {
		this.applicationContext = applicationContext;
		this.groovyClassLoader = groovyClassLoader;
		this.beanName = beanName;
	}

	public Object invoke(MethodInvocation methodInvocation) throws Throwable {
		Class targetClass = methodInvocation.getThis().getClass();
		Class groovyClass = groovyClassLoader.loadClass(targetClass.getName(), true, false);
		if(groovyClass == targetClass) {
			return methodInvocation.proceed();
		} else {
			applicationContext.refresh();
			Object newBean = applicationContext.getBean(beanName);
			Method originalMethod = methodInvocation.getMethod();
			Method newMethod = groovyClass.getMethod(originalMethod.getName(), originalMethod.getParameterTypes());
			return newMethod.invoke(newBean, methodInvocation.getArguments());
		}
	}

}
