package com.fg.scripting.groovy;

import com.fg.mock.*;
import com.fg.scripting.*;
import com.fg.scripting.groovy.classloader.*;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.support.DefaultListableBeanFactory;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import java.net.URL;

/**
 * Description
 *
 * @author Jan Novotný, FG Forrest a.s. (c) 2007
 * @version $Id: NativeSpringSupportTest.java,v 1.4 2010/04/17 20:27:25 u_novoj Exp $
 */
public class NativeSpringSupportTest extends AbstractGroovyTest {
	private ClassPathXmlApplicationContext ctx;

	@Override
	protected void setUp() throws Exception {
		super.setUp();
		DevelopmentGroovyFactory scriptGroovyFactory = new DevelopmentGroovyFactory(
				Thread.currentThread().getContextClassLoader(),
				new URL[] {rootFSR.getURL()},
				0,
				"utf-8"
		);

		ctx = new TestApplicationContext(
				new String[] {"META-INF/spring/spring-native-groovy.xml"},
				scriptGroovyFactory
		);

		ctx.setClassLoader(scriptGroovyFactory.getScriptingClassLoader());
		ctx.refresh();
	}

	public void testLoadGroovyClass() throws Exception {
		JavaHelloWorldInterface myGroovyInstance = (JavaHelloWorldInterface)ctx.getBean("mainGroovyClass");
		assertEquals("Hello Universe.", myGroovyInstance.sayHello());
	}

	public void testReloadLoadGroovyClass() throws Exception {
		JavaHelloWorldInterface myGroovyInstance = (JavaHelloWorldInterface)ctx.getBean("mainGroovyClass");
		assertEquals("Hello Universe.", myGroovyInstance.sayHello());

		modifyGroovyFile(
				"com/fg/mock/MainGroovyClass.groovy",
				"return HELLO_UNIVERSE;",
				"return \"Bye bye\";"
		);

		//refresh will occcur
		assertEquals("Bye bye", myGroovyInstance.sayHello());
	}

	public void testReloadLoadGroovyDependentClass() throws Exception {
		JavaHelloWorldInterface myGroovyInstance = (JavaHelloWorldInterface)ctx.getBean("referencingGroovyClass");
		assertEquals("Hello Universe.", myGroovyInstance.sayHello());

		modifyGroovyFile(
				"com/fg/mock/MainGroovyClass.groovy",
				"return HELLO_UNIVERSE;",
				"return \"Bye bye\";"
		);

		//no refresh will occcur - GroovyClassLoader is used - not GroovyScriptEngine - so the dependency checks are not
		//performed, this can by changed by writing custom ClassLoader based on GroovyScriptEngine
		assertEquals("Hello Universe.", myGroovyInstance.sayHello());
	}

	private static class TestApplicationContext extends ClassPathXmlApplicationContext {
		private ScriptingFactory scriptingFactory;

		private TestApplicationContext(String[] configLocations, ScriptingFactory scriptingFactory) throws BeansException {
			super(configLocations, false);
			this.scriptingFactory = scriptingFactory;
		}

		@Override
		protected DefaultListableBeanFactory createBeanFactory() {
			return new ScriptingBeanFactory(scriptingFactory);
		}

	}

}