package com.fg.scripting.groovy;

import com.fg.mock.*;
import com.fg.scripting.*;
import com.fg.scripting.groovy.classloader.*;

import java.net.URL;

/**
 * Description
 *
 * @author Jan Novotný, FG Forrest a.s. (c) 2007
 * @version $Id: ProductionGroovyFactoryTest.java,v 1.2 2010/01/14 12:30:27 u_novoj Exp $
 */
public class ProductionGroovyFactoryTest extends AbstractGroovyTest {
	private ScriptingFactory groovyFactory;

	@Override
	protected void setUp() throws Exception {
		super.setUp();
		groovyFactory = new ProductionGroovyFactory(
				Thread.currentThread().getContextClassLoader(),
				new SimpleGroovyResourceLoader(rootFSR),
				"utf-8"
		);
	}

	public void testRealWorldUsage() throws Exception {
		groovyFactory = new DevelopmentGroovyFactory(
				Thread.currentThread().getContextClassLoader(), //parent classloader
				new URL[] {rootFSR.getURL()}, //source codes
				0, //refresh interval
				"utf-8"
		);
		//create groovy instance, that
		MySpecificJavaClass groovyInstance = (MySpecificJavaClass)
				groovyFactory.createInstance("com.fg.mock.MainGroovyClass");
		/** uncomment this to set up custom intance factory
		 * (possibly intitializing the instance for example by dependency injection)
		 * ((ScriptingProxy)groovyInstance).setInstanceInitializer(... custom factory ...);
		 **/
		assertTrue(groovyInstance instanceof JavaHelloWorldInterface);
		assertEquals("Hello Universe.", ((JavaHelloWorldInterface)groovyInstance).sayHello());
	}

	public void testLoadGroovyClass() throws Exception {
		Class aClass = groovyFactory.loadClass("com.fg.mock.MainGroovyClass");
		assertNotNull(aClass);
	}

	public void testLoadJavaClass() throws Exception {
		Class aClass = groovyFactory.loadClass("java.lang.Object");
		assertNotNull(aClass);
	}

	public void testCreateGroovyInstance() throws Exception {
		JavaHelloWorldInterface groovyInstance = (JavaHelloWorldInterface)
				groovyFactory.createInstance("com.fg.mock.MainGroovyClass");
		assertEquals("Hello Universe.", groovyInstance.sayHello());
	}

	public void testCreateGroovyInstanceReload() throws Exception {
		JavaHelloWorldInterface groovyInstance = (JavaHelloWorldInterface)
				groovyFactory.createInstance("com.fg.mock.MainGroovyClass");
		assertEquals("Hello Universe.", groovyInstance.sayHello());
		assertTrue(groovyInstance instanceof MySpecificJavaClass);
		//we'll change source file contents so as method returns different value
		modifyGroovyFile(
				"com/fg/mock/MainGroovyClass.groovy",
				"return HELLO_UNIVERSE;",
				"return \"Bye bye\";"
		);
		//now behaviour thanks to our proxy get changed even on the same object reference.
		assertEquals("Hello Universe.", groovyInstance.sayHello());
	}

}