package com.fg.scripting.groovy;

import com.fg.mock.JavaHelloWorldInterface;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * Description
 *
 * @author Jan Novotný, FG Forrest a.s. (c) 2007
 * @version $Id: SpringGroovyReloadSupportTest.java,v 1.3 2009/11/15 20:05:10 u_novoj Exp $
 */
public class SpringGroovyReloadSupportTest extends AbstractGroovyTest {
	private ClassPathXmlApplicationContext ctx;

	@Override
	protected void setUp() throws Exception {
		super.setUp();
		ctx = new ClassPathXmlApplicationContext(
				"META-INF/spring/spring-groovy.xml"
		);
	}

	public void testReloadLoadGroovyClass() throws Exception {
		JavaHelloWorldInterface myGroovyInstance = (JavaHelloWorldInterface)ctx.getBean("mainGroovyClass");
		assertEquals("Hello Universe.", myGroovyInstance.sayHello());

		modifyGroovyFile(
				"com/fg/mock/MainGroovyClass.groovy",
				"return HELLO_UNIVERSE;",
				"return \"Bye bye\";"
		);

		//Spring will take care of refresh
		assertEquals("Bye bye", myGroovyInstance.sayHello());
	}

	public void testReloadLoadGroovyDependentClass() throws Exception {
		JavaHelloWorldInterface myGroovyInstance = (JavaHelloWorldInterface)ctx.getBean("mainGroovyClass");
		assertEquals("Hello Universe.", myGroovyInstance.sayHello());

		modifyGroovyFile(
				"com/fg/mock/GroovyHelloWorldInterface.groovy",
				"HELLO_UNIVERSE = \"Hello Universe.\";",
				"HELLO_UNIVERSE = \"Bye Universe.\";"
		);

		//Spring DOESN'T take dependent classes into an account
		assertEquals("Hello Universe.", myGroovyInstance.sayHello());
	}

	public void testReloadGroovyDependentClassVerifyInstance() throws Exception {
		JavaHelloWorldInterface myGroovyInstance = (JavaHelloWorldInterface)ctx.getBean("referencingGroovyClass");
		assertEquals("Hello Universe.", myGroovyInstance.sayHello());

		modifyGroovyFile(
				"com/fg/mock/MainGroovyClass.groovy",
				"return HELLO_UNIVERSE;",
				"return \"Hei universumissa.\";"
		);

		//Spring can refresh underlying bean even for existing instances (proxy plays its role)
		assertEquals("Hei universumissa.", myGroovyInstance.sayHello());
	}

}